function map = mapdissim(im,ws,hsz,zp)

% Fast calculation of dissimilarity map
%
% (c) M.Ponomarenko, 2017
%
% map = mapdissim(image,ws,hsz,zp);
% 
% Inputs:
% im  - grayscale image
% ws  - patch size for dissimilarity calculation
% hsz - half of size of search zone of similar patches
% zp  - size of area around of each patch which will be excluded from 
%       searching to provide robustness to spatially correlated noise. 
%       Set zp=0 if the noise is white or there is no noise on the image.
%
% Output:
% map - values of dissimilarity for each possible position of patch [ws ws] pixels
%
% Examples:
% map = mapdissim(im,9,6,0);
% map = mapdissim(im);

  im = double(im);
  [loy,lox]=size(im);
  bige=1e30;

  if nargin < 4
    zp=0;
  end

  if nargin < 3
    hsz=11;
  end

  if nargin < 2
    ws=9;
  end
   
  a=zeros(loy+hsz*2-1,lox+hsz*2-1);    % zeropadding copy of im
  a(hsz+1:hsz+loy,hsz+1:hsz+lox)=im;

  map=zeros(loy,lox)+bige;       % starting dissimilarity
  
  H = fspecial('average', ws);    % patch size 

  for i=1:hsz*2
    for j=1:hsz*2
      if abs(i-hsz-1)>zp || abs(j-hsz-1)>zp 
        delt=(a(i:i+loy-1,j:j+lox-1)-im).^2; % pixel based MSE
        delt=imfilter(delt,H,'replicate');   % mean MSE in window [ws ws]
        map=min(map,delt);
      end
    end
  end

end
